#include "config.h"

#include <string.h>

#include <gmodule.h>

#include "pango-backend.h"

#include "pango-utils.h"

void (*render_layout) (PangoBitmap *bitmap,  
                       PangoLayout *layout,
                       int          x, 
                       int          y) = NULL;

static GModule *
open_guessed_module (const char *backend)
{
  GModule    *module;
  char       *module_name;
  const char *pango_name = "pango-" PANGO_VERSION ".dll";

  /* guess the module name from the backend string */
  module_name = g_strdup_printf ("%s\\pango%s-%s",
                                 g_win32_get_package_installation_directory (NULL, pango_name),
                                 backend, PANGO_VERSION);

  module = g_module_open (module_name, 0);

  if (!module)
    {
      g_print ("Cannot load module %s: %s\n",
               module_name, g_module_error());

      g_free (module_name);
      /* try without version name */
      module_name = g_strdup_printf ("pango%s", backend);
      module = g_module_open (module_name, 0);
    }
  g_free (module_name);

  return module;
}

/**
 * pango_backend_get_context:
 * @backend: the name of the prefered backend, i.e. 'x', 'ft2', 'win32' or
 *           %NULL if none is prefered
 *
 * Retrieves a #PangoContext appropriate for rendering with Pango fonts.
 * If the prefered backend isn't available the function tries to return 
 * a #PangoContext from another backend.
 * 
 * Return value: the new #PangoContext or #NULL ...
 **/
PangoContext *
pango_backend_get_context (const char* backend)
{
  GModule *module = NULL;
  char    *backends [] = {"x", "ft2", "win32"};
  int      n_backend;
  PangoContext *context = NULL;

  if (backend)
    module = open_guessed_module (backend);

  /* try already registered backends ? */

  /* try defaults */
  if (!module)
    {
       for (n_backend = 0; n_backend < G_N_ELEMENTS (backends); n_backend++)
         {
           if (0 == strcmp (backend, backends[n_backend]))
             continue; /* already failed above */
           
           module = open_guessed_module (backends[n_backend]);
           if (module)
             {
               backend = backends[n_backend];
               break;
             }
         }
    }

  if (module)
    {
      char *symbol;
      PangoContext *(* get_context) (void); //FIXME: X needs a display pointer

      symbol = g_strdup_printf ("pango_%s_get_context", backend);

      g_module_symbol (module, symbol, (gpointer)&get_context);

      if (get_context)
        context = (*get_context) ();
      else
        g_print ("Can't get symbol %s!", symbol);

      g_free (symbol);
      symbol = g_strdup_printf ("pango_%s_backend_render_layout", backend);

      g_module_symbol (module, symbol, (gpointer)&render_layout);
    }

  return context;
}

/**
 * pango_backend_render_layout:
 * @bitmap:    a #PangoBitmap to render the line onto
 * @layout:    a #PangoLayout
 * @x:         the X position of the left of the layout (in pixels)
 * @y:         the Y position of the top of the layout (in pixels)
 *
 * Render a #PangoLayoutLine onto a FreeType2 bitmap
 */
void 
pango_backend_render_layout (PangoBitmap *bitmap,
                             PangoLayout *layout,
                             int          x, 
                             int          y)
{
  if (!render_layout)
    g_warning ("pango_??_backend_render_layout () not available!");
  else
    (*render_layout) (bitmap, layout, x, y);
}
